#include "jserver.h"


sreturn js_service_prescreen(jpacket p)
{
    static mmaster om = NULL;
    udata user;
    char *u, *ul;
    jid test;

    if(om == NULL) om = js_mapi_master(P_OFFLINE);

    if(p == NULL) return S_IGNORE;

    /* enforce the username to lowercase for registration */
    u = xmlnode_get_tag_data(p->iq,"username");
    if(u != NULL)
    {
        for(ul = u;*ul != '\0'; ul++)
            *ul = tolower(*ul);

        test = jid_new(p->p,"test");
        jid_set(test,u,JID_USER);
        if(test->user == NULL)
        {
            jutil_error(p->x, TERROR_NOTACCEPTABLE);
            return S_BOUNCE;
        }
    }

    if(p->type == JPACKET_IQ && NSCHECK(p->iq,NS_AUTH))
    {
        log_debug(ZONE,"auth request");

        user = js_user(u);
        if(user == NULL || xmlnode_get_tag_data(p->iq,"resource") == NULL)
        { /* no user on this server or no resource sent */
            jutil_error(p->x, TERROR_AUTH);
        }else{
            if(js_mapi_call(P_OFFLINE, om->l, p, user, NULL, MAPI_VARAUTH))
            { /* someone handled it */
                jpacket_reset(p);
                if(jpacket_subtype(p) == JPACKET__RESULT)
                    return S_AUTHED;
            }else{ /* nobody understood the auth */
                jutil_error(p->x, TERROR_INTERNAL);
            }
        }
    }else if(p->type == JPACKET_IQ && NSCHECK(p->iq,NS_REGISTER)){
        log_debug(ZONE,"registration request");

        if(!js_mapi_call(P_OFFLINE, om->l, p, NULL, NULL, MAPI_VARREGISTER))
            jutil_error(p->x, TERROR_NOTIMPL);
    }else{
        jutil_error(p->x, TERROR_AUTH);
    }

    return S_BOUNCE;
}

/*
 *  js_service_listen -- start the listen thread(s) for the service based on the configuration
 *
 *  parameters
 *  	svc -- the name of the service
 *
 */
void js_service_listen(char *svc, tlisten_onConnect evt)
{

    xmlnode x;  /* data from the config */
    char *ip;	/* IP address or hostname to listen on */
    int port;	/* port to listen on */
    int flag = 0;

    /* scan the config file listen entries */
    for(x = xmlnode_get_firstchild(js_config("listen")); x != NULL; x = xmlnode_get_nextsibling(x))
    {
        if(j_strcmp(xmlnode_get_name(x),svc) != 0) continue;

        /* attempt to get the port from the node data */
        if(xmlnode_get_data(x) == NULL || sscanf(xmlnode_get_data(x), "%d", &port) == 0) continue;

        /* get the hostname from the ip attribute if available */
        ip = xmlnode_get_attrib(x,"ip");
 
        /* create thread to listen on the host and port */
        if(tlisten_new(port, ip, evt, NULL) == NULL)
        {
            log_error("jserver","failed to listen on %s:%d",ip,port);
        }else{
            log_debug(ZONE,"listening on %s:%d",ip,port);
            flag = 1;
        }
    }

    if(flag == 0)
        log_error("jserver","Failed to listen for service %s",svc);
}
